<?php

namespace Drupal\synpay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Component\Serialization\Json;

/**
 * Synpay Payment Gateway.
 *
 * @CommercePaymentGateway(
 *   id = "synpay",
 *   label = @Translation("Synpay Gateway"),
 *   display_label = @Translation("Synpay Gateway"),
 *   forms = {
 *     "offsite-payment" = "Drupal\synpay\PluginForm\OffsiteRedirect\OffsitePaymentForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   requires_billing_information = FALSE,
 *   credit_card_types = {
 *     "maestro", "mastercard", "visa", "mir",
 *   },
 * )
 */
class SynpayPaymentGateway extends OffsitePaymentGatewayBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $manager = \Drupal::service('plugin.manager.synpay');
    $form['gateway'] = [
      '#title' => $this->t("Gateway"),
      '#type' => 'radios',
      '#required' => TRUE,
      '#default_value' => $this->configuration['gateway'],
    ];
    foreach ($manager->getDefinitions() as $gateway => $source) {
      $form['gateway']['#options'][$gateway] = $source['title'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue($form['#parents']);
    if (empty($values['gateway'])) {
      $form_state->setError($form['gateway'], $this->t("Gateway is required."));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['gateway'] = $values['gateway'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $payment_gateway_plugin = $this->parentEntity->getPlugin();
    $configs = $payment_gateway_plugin->getConfiguration();
    $synpay = \Drupal::service('synpay.gateway')
      ->init($configs['mode'], $configs);
      // $remote_id = $request->query->all() ?? '';
    $order_status = $synpay->onReturn($order, $request);
    if ($order_status != 'completed') {
      $message = $this->t("Payment failed!");
      \Drupal::messenger()->addError($message);
      throw new PaymentGatewayException('');
    }
  }

  /**
   * Processes the notification request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Symfony\Component\HttpFoundation\Response|null
   *   The response, or NULL to return an empty HTTP 200 response.
   */
  public function onNotify(Request $request) {
    $payment_gateway_plugin = $this->parentEntity->getPlugin();
    $configs = $payment_gateway_plugin->getConfiguration();
    $synpay = \Drupal::service('synpay.gateway')
      ->init($configs['mode'], $configs);
    $remote_id = $request->query->all() ?? '';
    $response = $synpay->onNotify($request);
    if ($response) {
      return $response;
    }
    return new Response('OK', 200);
  }

  /**
   * Query Payment.
   */
  private function query(string $remote_id, $order_id = FALSE) {
    /** @var \Drupal\commerce_payment\PaymentStorageInterface $payment_storage */
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    if (empty($remote_id) && $order_id) {
      $payments = $payment_storage->loadByProperties(['order_id' => $order_id]);
      if ($payments) {
        $payment = end($payments);
      }
    }
    else {
      $payment = $payment_storage->loadByRemoteId($remote_id);
    }
    return $payment;
  }

}
