<?php

namespace Drupal\synpay\Plugin\Synpay;

use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Ramsey\Uuid\Uuid;
// use GuzzleHttp\Client;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_checkout\Event\CheckoutEvents;
use Drupal\commerce_order\Event\OrderEvent;
use GuzzleHttp\Exception\ClientException;
use Drupal\Component\Serialization\Json;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides Tinkoff Gateway.
 *
 * @SynpayAnnotation(
 *   id = "paykeeper",
 *   title = @Translation("Pay Keeper"),
 * )
 */
class SynpayPayKeeper extends SynpayPluginBase implements SynpayPluginInterface {
  const PAYMENT_EXPIRATION_PERIOD = '+1 hour';
  const LABEL = '"Pay Keeper эквайринг"';
  const HOST = "https://cledelame.server.paykeeper.ru";
  const HOST_TEST = "https://demo.paykeeper.ru";
  // TRUE - цена с копейками, FALSE - цена в копейках.
  const PRECISION = TRUE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->plugin_id = $plugin_id;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->host = \Drupal::request()->getSchemeAndHttpHost();
    $this->currency = 'RUB';
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    if ($this->active == 1) {
      $mode = $this->config->get("{$plugin_id}_test_mode");
      switch ($mode) {
        case 0:
          $this->api = self::HOST;
          break;

        case 1:
          $this->api = self::HOST_TEST;
          break;
      }
      $this->client = \Drupal::httpClient();
      $this->options = [
        'headers' => [
          'Content-type' => 'application/x-www-form-urlencoded',
          // 'Cache-Control' => 'no-cache',
        ],
        'auth' => [trim($this->user), trim($this->pass)],
      ];
      $url = "{$this->api}/info/settings/token/";
      $token = '';
      try {
        $response = $this->client->get($url, $this->options);
        $code = $response->getStatusCode();
        if ($code == 200) {
          $answer = $response->getBody()->getContents();
          $this->token = Json::decode($answer)['token'];
        }
      }
      catch (ClientException $exception) {
        $message = $exception->getMessage();
        \Drupal::messenger()->addError($message);
      }
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $order = Uuid::uuid4()->getHex()->toString();
    $data = $this->options;
    $data['form_params']['pay_amount'] = $total;
    $data['form_params']['orderid'] = $order;
    $data['form_params']['token'] = $this->token;
    $url = "{$this->api}/info/options/byid/";
    try {
      $response = $this->client->post($url, $data);
      $code = $response->getStatusCode();
      $answer = $response->getBody()->getContents();
      $result = Json::decode($answer);
    }
    catch (ClientException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    $data = $this->options;
    $data['form_params']['orderid'] = $order_id;
    $data['form_params']['pay_amount'] = $order_amount;
    // $data['form_params']['user_result_callback'] = $params['returnUrl'];
    $data['form_params']['token'] = $this->token;
    $url = "{$this->api}/change/invoice/preview/";
    try {
      $answer = $this->client->post($url, $data);
      $code = $answer->getStatusCode();
      if ($code == 200) {
        $body = $answer->getBody()->getContents();
        $response = JSON::decode($body);
        if (isset($response['invoice_id'])) {
          return [
            'orderId' => $response['invoice_id'],
            'formUrl' => $response['invoice_url'],
          ];
        }
        else {
          \Drupal::messenger()->addError($response['Message']);
          \Drupal::messenger()->addError($response['Details']);
          return FALSE;
        }
      }
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      // \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
    return FALSE;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $paymentInfo = \Drupal::request()->request->all();

    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['payment_id' => $paymentInfo['orderid']]);
    $payment = reset($payments);
    if (!$payment) {
      return new Response('Bad request', 400);
    }
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }
    $key = md5($paymentInfo['id'] . number_format($paymentInfo['sum'], 2, ".", "") . $paymentInfo['orderid'] . $this->secure);
    if ($key == $paymentInfo['key']) {
      $payment->state = 'completed';
      $payment->setCompletedTime(time());
      $payment->setRemoteState('paid');
      $payment->save();
      if ($order->state->value != 'completed') {
        $event = new OrderEvent($order);
        $eventDispatcher = \Drupal::service('event_dispatcher');
        $eventDispatcher->dispatch(CheckoutEvents::COMPLETION, $event);
        $order->getState()->applyTransitionById('place');
        $order->save();
      }
    }

    $answer = md5($paymentInfo['id'] . $this->secure);
    return new Response("OK $answer", 200);
  }

  /**
   * Commerce Return.
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $result = 'completed';
    return $result;
  }

  /**
   * Check Payments.
   */
  private function checkPayment($id) {
    $params = $this->options;
    $params['PaymentId'] = $id;
    $data = [
      'Password' => $this->pass,
      'PaymentId' => $id,
      'TerminalKey' => $this->user,
    ];
    $token = '';
    foreach ($data as $key => $value) {
      $token .= $value;
    }
    $hash = hash('sha256', $token);
    $params['Token'] = $hash;
    try {
      $response = $this->client->post("{$this->api}GetState", ['json' => $params]);
      $code = $response->getStatusCode();
      $answer = Json::decode($response->getBody()->getContents());
      return $answer;
    }
    catch (ClientException $exception) {
      // $message = $exception->getMessage();
      // \Drupal::messenger()->addError($message);
    }
    return FALSE;
  }

  /**
   * UpdatePayment.
   */
  private function updatePayment($payment, $status) {
    if (!$status) {
      $payment->state = 'authorization_voided';
      $payment->setRemoteState('Error');
      $result = 'authorization_voided';
    }
    else {
      switch ($status['Status']) {
        case 'CONFIRMED':
          $payment->state = 'completed';
          $payment->setCompletedTime(time());
          $payment->setRemoteState($status['Status']);
          $result = 'completed';
          break;

        default:
          $payment->state = 'authorization_voided';
          $payment->setRemoteState($status['Status']);
          $result = 'authorization_voided';
          break;
      }
    }
    $payment->save();
    return $result;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayPayKeeper')->info($message);
  }

}
