<?php

namespace Drupal\synpay\Plugin\Synpay;

use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Ramsey\Uuid\Uuid;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_checkout\Event\CheckoutEvents;
use Drupal\commerce_order\Event\OrderEvent;
use GuzzleHttp\Exception\ClientException;
use Drupal\Component\Serialization\Json;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides Tinkoff Gateway.
 *
 * @SynpayAnnotation(
 *   id = "tinkoff",
 *   title = @Translation("Tinkoff"),
 * )
 */
class SynpayTinkoff extends SynpayPluginBase implements SynpayPluginInterface {
  const PAYMENT_EXPIRATION_PERIOD = '+1 hour';
  const LABEL = '"Тинькофф эквайринг"';
  const HOST = "https://securepay.tinkoff.ru/v2/";
  const HOST_TEST = "https://securepay.tinkoff.ru/v2/";
  // TRUE - цена с копейками, FALSE - цена в копейках.
  const PRECISION = FALSE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->plugin_id = $plugin_id;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->host = \Drupal::request()->getSchemeAndHttpHost();
    $this->currency = 'RUB';
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    if ($this->active == 1) {
      $mode = $this->config->get("{$plugin_id}_test_mode");
      switch ($mode) {
        case 0:
          $this->api = self::HOST;
          break;

        case 1:
          $this->api = self::HOST_TEST;
          break;
      }
      $this->client = \Drupal::httpClient();
      $this->options = [
        'headers' => [
          'Content-type' => 'application/x-www-form-urlencoded',
          'Cache-Control' => 'no-cache',
        ],
        'TerminalKey' => trim($this->user),
      ];
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $order = Uuid::uuid4()->getHex()->toString();
    $order_amount = bcmul($total, 100, 0);
    $params = $this->options;
    $params['Amount'] = $order_amount;
    $params['OrderId'] = $order;
    $params['SuccessURL'] = "{$this->host}/synpay/return_direct/$this->plugin_id";
    $params['FailURL'] = "{$this->host}/synpay/return_direct/$this->plugin_id?fail";
    try {
      $response = $this->client->post("{$this->api}Init", ['json' => $params]);
      $code = $response->getStatusCode();
      $result = Json::decode($response->getBody()->getContents());
    }
    catch (ClientException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    $payments = \Drupal::entityTypeManager()->getStorage('commerce_payment')
      ->loadByProperties(['payment_id' => $order_id]);
    $payment = reset($payments);
    $data = $this->options;
    $data['OrderId'] = $order_id;
    $data['Amount'] = $order_amount;
    $data['SuccessURL'] = $params['returnUrl'];
    $data['FailURL'] = $params['failUrl'];
    $data['NotificationURL'] = "{$this->host}/payment/notify/{$payment->getPaymentGatewayId()}";
    $data['Receipt']['Taxation'] = 'osn';
    if (!empty($params['orderBundle']['customerDetails']['email'])) {
      $data['Receipt']['Email'] = $params['orderBundle']['customerDetails']['email'];
    }
    foreach ($params['orderBundle']['cartItems']['items'] as $item) {
      $data['Receipt']['Items'][] = [
        'Name' => $item['name'],
        'Quantity' => $item['quantity']['value'],
        'Price' => $item['itemPrice'],
        'Amount' => $item['itemAmount'],
        'Tax' => 'vat20',
      ];
    }
    $url = "{$this->api}Init";
    try {
      $answer = $this->client->request('POST', $url, ['json' => $data]);
      $code = $answer->getStatusCode();
      if ($code == 200) {
        $body = $answer->getBody()->getContents();
        $response = JSON::decode($body);
        if ($response['Success']) {
          return [
            // 'data' => $data,
            'orderId' => $response['PaymentId'],
            'formUrl' => $response['PaymentURL'],
          ];
        }
        else {
          \Drupal::messenger()->addError($response['Message']);
          \Drupal::messenger()->addError($response['Details']);
          return FALSE;
        }
      }
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
    return FALSE;
  }

  /**
   * Commerce Return.
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $paymentInfo = Json::decode(
      file_get_contents('php://input')
    );
    if (!empty($paymentInfo['PaymentId'])) {
      $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
      $payment = $payment_storage->loadByRemoteId($paymentInfo['PaymentId']);
      $status = $this->checkPayment($paymentInfo['PaymentId']);
      $result = $this->updatePayment($payment, $status);
    }
    else {
      $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
      $payments = $payment_storage->loadByProperties(['order_id' => $order->id()]);
      $success = FALSE;
      foreach ($payments as $payment) {
        $status = $this->checkPayment($payment->remote_id->value);
        $answer = $this->updatePayment($payment, $status);
        if ($answer == 'completed' && !$success) {
          $success = TRUE;
        }
      }
      $result = $success ? 'completed' : $answer;
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $paymentInfo = Json::decode(
      file_get_contents('php://input')
    );
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['remote_id' => $paymentInfo['PaymentId']]);
    $payment = reset($payments);
    if (!$payment) {
      return new Response('Bad request', 400);
    }
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }
    switch ($paymentInfo['Status']) {
      case 'CONFIRMED':
      case 'AUTHORIZED':
        $payment->state = 'completed';
        $payment->setCompletedTime(time());
        $event = new OrderEvent($order);
        $eventDispatcher = \Drupal::service('event_dispatcher');
        $eventDispatcher->dispatch(CheckoutEvents::COMPLETION, $event);
        $order->getState()->applyTransitionById('place');
        $order->save();
        break;

      case 'REJECTED':
        $payment->state = 'authorization_voided';
        break;
    }
    $payment->setRemoteState($paymentInfo['Status']);
    $payment->save();

    return new Response('OK', 200);
  }

  /**
   * Check Payments.
   */
  private function checkPayment($id) {
    $params = $this->options;
    $params['PaymentId'] = $id;
    $data = [
      'Password' => $this->pass,
      'PaymentId' => $id,
      'TerminalKey' => $this->user,
    ];
    // array_multisort($data, SORT_DESC, SORT_STRING, $data);
    $token = '';
    foreach ($data as $key => $value) {
      $token .= $value;
    }
    $hash = hash('sha256', $token);
    $params['Token'] = $hash;
    try {
      $response = $this->client->post("{$this->api}GetState", ['json' => $params]);
      $code = $response->getStatusCode();
      $answer = Json::decode($response->getBody()->getContents());
      return $answer;
    }
    catch (ClientException $exception) {
      // $message = $exception->getMessage();
      // \Drupal::messenger()->addError($message);
    }
    return FALSE;
  }

  /**
   * UpdatePayment.
   */
  private function updatePayment($payment, $status) {
    if (!$status) {
      $payment->state = 'authorization_voided';
      $payment->setRemoteState('Error');
      $result = 'authorization_voided';
    }
    else {
      switch ($status['Status']) {
        case 'CONFIRMED':
        case 'AUTHORIZED':
          $payment->state = 'completed';
          $payment->setCompletedTime(time());
          $payment->setRemoteState($status['Status']);
          $result = 'completed';
          break;

        default:
          $payment->state = 'authorization_voided';
          $payment->setRemoteState($status['Status']);
          $result = 'authorization_voided';
          break;
      }
    }
    $payment->save();
    return $result;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayTinkoff')->info($message);
  }

}
